# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 Avencall
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

import logging
import time

from xivo_dao import session_dao, admin_dao
from xivo_dao.helpers.db_utils import session_scope

from xivo_auth import BaseAuthenticationBackend
from phpserialize import *

logger = logging.getLogger(__name__)


class XiVOSession(BaseAuthenticationBackend):

    def get_acls(self, login, args):
        with session_scope():
            username = self._get_username(args['cookie'])
            if self._is_root(username):
                return ['confd.#']
            else:
                return ['confd.label.#']

    def get_ids(self, username, args):
        with session_scope():
            username = self._get_username(args['cookie'])
            auth_id = str(admin_dao.get_admin_id(username))
        user_uuid = None
        return auth_id, user_uuid

    def verify_password(self, login, password, args):
        cookie = args['cookie']
        if cookie is not None:
            with session_scope():
                current_timestamp = int(time.time())
                session_obj = session_dao.get_session(args['cookie'])
                if current_timestamp < session_obj.expire and self._deserialize_username(session_obj.data) is not None:
                    return True
                else:
                    return False
        else:
            return False

    def _get_username(self, cookie):
        session_obj = session_dao.get_session(cookie)
        username = self._deserialize_username(session_obj.data)
        logger.debug("Retrieved username %s " % username)
        return username if username is not None else None

    @staticmethod
    def _deserialize_username(session_data):
        try:
            serialized_data = session_data.split('_USR|', 5)[1].split("_WEBI", 5)[0]
            deserialized_user = loads(serialized_data, object_hook=phpobject)
            return deserialized_user.login
        except Exception, e:
            logger.error("Failed to deserialize session data: %s " % str(e))
            return None

    @staticmethod
    def _is_root(username):
        if username == 'root':
            return True
        elif username is None:
            raise ValueError('username must not be None')
        else:
            return False
